package fixstyle;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;

public final class DirectoryRecursor implements FilenameFilter
{
    private Queue queue;

    public boolean accept(File dir, String name)
    {
        return true;
    }
    
    public boolean accept(String name)
    {
        int len = name.length();
        
        return (name.substring(len-5).equals(".java"));
    }

    /**
     * Calls {@link #scanDirectory(File, String, int)}.
     *
     * @param dir directory to process.
     */
    public File[] scanDirectory(String path) throws IOException
    {
        queue = new Queue();

        File dir = new File(path);

        scanDirectory(dir, path);

        return queue.toArray();
    }

    /**
     * Searches recursively all directories.
     *
     * @param dir directory to search.
     * @param rootDir root directory of the search.
     */
    private void scanDirectory(File dir, String rootDir)
        throws IOException
    {
        File[] dirEntries = dir.listFiles(this);

        if (dirEntries != null)
        {
            for (int i = 0; i < dirEntries.length; i++)
            {
                File f = dirEntries[i];
                // recursion one level up
                if (f.isDirectory())
                {
                    scanDirectory(f, rootDir);
                } else
                {
                    if (accept(f.getName()))
                    {
                        queue.push(f);
                    }
                }
            }
        }
    }

    //~ Inner Classes --------------------------------------------------------------------

    /**
     * Stores all found directories/files.
     */
    private class Queue
    {
        LinkedList list;

        /**
         * Creates a new Queue object.
         */
        public Queue()
        {
            this.list = new LinkedList();
        }

        private Queue(LinkedList list)
        {
            this.list = new LinkedList(list);
        }

        public boolean isEmpty()
        {
            return this.list.isEmpty();
        }

        public Iterator iterator()
        {
            Queue stack = new Queue(this.list);

            return stack.list.iterator();
        }

        public File pop()
        {
            return (File) this.list.removeLast();
        }

        public void push(File file)
        {
            this.list.add(file);
        }

        public int size()
        {
            return this.list.size();
        }

        public File[] toArray()
        {
            Queue stack = new Queue(this.list);
            File[] files = new File[stack.list.size()];
            stack.list.toArray(files);

            return files;
        }
    }
}
