package fixstyle;

import java.util.ArrayList;

import parser.ASTAdditiveExpression;
import parser.ASTAllocationExpression;
import parser.ASTAndExpression;
import parser.ASTArgumentList;
import parser.ASTArguments;
import parser.ASTArrayDimsAndInits;
import parser.ASTArrayInitializer;
import parser.ASTAssignmentOperator;
import parser.ASTBlock;
import parser.ASTBlockStatement;
import parser.ASTBooleanLiteral;
import parser.ASTBreakStatement;
import parser.ASTCastExpression;
import parser.ASTCastLookahead;
import parser.ASTClassBody;
import parser.ASTClassBodyDeclaration;
import parser.ASTClassDeclaration;
import parser.ASTCompilationUnit;
import parser.ASTConditionalAndExpression;
import parser.ASTConditionalExpression;
import parser.ASTConditionalOrExpression;
import parser.ASTConstructorDeclaration;
import parser.ASTContinueStatement;
import parser.ASTDoStatement;
import parser.ASTEmptyStatement;
import parser.ASTEqualityExpression;
import parser.ASTExclusiveOrExpression;
import parser.ASTExplicitConstructorInvocation;
import parser.ASTExpression;
import parser.ASTFieldDeclaration;
import parser.ASTForInit;
import parser.ASTForStatement;
import parser.ASTForUpdate;
import parser.ASTFormalParameter;
import parser.ASTFormalParameters;
import parser.ASTIfStatement;
import parser.ASTImportDeclaration;
import parser.ASTInclusiveOrExpression;
import parser.ASTInitializer;
import parser.ASTInstanceOfExpression;
import parser.ASTInterfaceDeclaration;
import parser.ASTInterfaceMemberDeclaration;
import parser.ASTLabeledStatement;
import parser.ASTLiteral;
import parser.ASTLocalVariableDeclaration;
import parser.ASTMethodDeclaration;
import parser.ASTMethodDeclarationLookahead;
import parser.ASTMethodDeclarator;
import parser.ASTMultiplicativeExpression;
import parser.ASTName;
import parser.ASTNameList;
import parser.ASTNestedClassDeclaration;
import parser.ASTNestedInterfaceDeclaration;
import parser.ASTNullLiteral;
import parser.ASTPackageDeclaration;
import parser.ASTPostfixExpression;
import parser.ASTPreDecrementExpression;
import parser.ASTPreIncrementExpression;
import parser.ASTPrimaryExpression;
import parser.ASTPrimaryPrefix;
import parser.ASTPrimarySuffix;
import parser.ASTPrimitiveType;
import parser.ASTRelationalExpression;
import parser.ASTResultType;
import parser.ASTReturnStatement;
import parser.ASTShiftExpression;
import parser.ASTStatement;
import parser.ASTStatementExpression;
import parser.ASTStatementExpressionList;
import parser.ASTSwitchLabel;
import parser.ASTSwitchStatement;
import parser.ASTSynchronizedStatement;
import parser.ASTThrowStatement;
import parser.ASTTryStatement;
import parser.ASTType;
import parser.ASTTypeDeclaration;
import parser.ASTUnaryExpression;
import parser.ASTUnaryExpressionNotPlusMinus;
import parser.ASTUnmodifiedClassDeclaration;
import parser.ASTUnmodifiedInterfaceDeclaration;
import parser.ASTVariableDeclarator;
import parser.ASTVariableDeclaratorId;
import parser.ASTVariableInitializer;
import parser.ASTWhileStatement;
import parser.JavaParserVisitor;
import parser.SimpleNode;
import parser.Token;

public class ModifiableTokenStreamMaker implements JavaParserVisitor
{
    private ArrayList list; // = new ArrayList();

    public ModifiableTokenStreamMaker(ArrayList list)
    {
        this.list = list;
    }

    public Object print(parser.SimpleNode node, Object data)
    {
        Token t1 = node.getFirstToken();
        Token t = new Token();
        t.next = t1;

        SimpleNode n;
        for (int ord = 0; ord < node.jjtGetNumChildren(); ord++)
        {
            n = (SimpleNode) node.jjtGetChild(ord);
            while (true)
            {
                t = t.next;
                if (t == n.getFirstToken())
                {
                    break;
                }
                print(t);
            }
            n.jjtAccept(this, data);
            t = n.getLastToken();
        }

        while (t != node.getLastToken())
        {
            t = t.next;
            print(t);
        }
        return data;
    }

    protected void print(Token t)
    {
        Token tt = t.specialToken;
        if (tt != null)
        {
            while (tt.specialToken != null)
            {
                tt = tt.specialToken;
            }
            while (tt != null)
            {
                //if (tt.image.equals("public")) {
                //out.print("!!!!");
                //}
                // out.print("[" + addUnicodeEscapes(tt.image) + "]");
                list.add(new XTokenWhiteSpace(tt.image));
                //out.print(addUnicodeEscapes(tt.image));
                tt = tt.next;
            }
        }
        //out.print("<" + addUnicodeEscapes(t.image) + ">" + t.toString());
        list.add(new XTokenReal(t.image));
        //out.print(addUnicodeEscapes(t.image));
    }

    String addUnicodeEscapes(String str)
    {
        String retval = "";
        char ch;
        for (int i = 0; i < str.length(); i++)
        {
            ch = str.charAt(i);
            if ((ch < 0x20 || ch > 0x7e)
                && ch != '\t'
                && ch != '\n'
                && ch != '\r'
                && ch != '\f')
            {
                String s = "0000" + Integer.toString(ch, 16);
                retval += "\\u" + s.substring(s.length() - 4, s.length());
            } else
            {
                retval += ch;
            }
        }
        return retval;
    }

    public Object visit(SimpleNode node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTCompilationUnit node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTPackageDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTImportDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTTypeDeclaration node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTClassDeclaration node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTUnmodifiedClassDeclaration node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTClassBody node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTNestedClassDeclaration node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTClassBodyDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTMethodDeclarationLookahead node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTInterfaceDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTNestedInterfaceDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTUnmodifiedInterfaceDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTInterfaceMemberDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTFieldDeclaration node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTVariableDeclarator node, Object data)
    {
        //print(node, data); //TODO
        return print(node, data);
    }

    public Object visit(ASTVariableDeclaratorId node, Object data)
    {
        //print(node, data); //TODO
        return print(node, data);
    }

    public Object visit(ASTVariableInitializer node, Object data)
    {
        //print(node, data); //TODO
        return print(node, data);
    }

    public Object visit(ASTArrayInitializer node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTMethodDeclaration node, Object data)
    {
        //print(node, data);//TODO  <--
        return print(node, data);
    }

    public Object visit(ASTMethodDeclarator node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTFormalParameters node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTFormalParameter node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTConstructorDeclaration node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTExplicitConstructorInvocation node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTInitializer node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTType node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTPrimitiveType node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTResultType node, Object data)
    {
        //print(node, data);//TODO // includes the "void"...
        return print(node, data);
    }

    public Object visit(ASTName node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTNameList node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTExpression node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTAssignmentOperator node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTConditionalExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTConditionalOrExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTConditionalAndExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTInclusiveOrExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTExclusiveOrExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTAndExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTEqualityExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTInstanceOfExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTRelationalExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTShiftExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTAdditiveExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTMultiplicativeExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTUnaryExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTPreIncrementExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTPreDecrementExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTUnaryExpressionNotPlusMinus node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTCastLookahead node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTPostfixExpression node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTCastExpression node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTPrimaryExpression node, Object data)
    {
        // print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTPrimaryPrefix node, Object data)
    {
        // print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTPrimarySuffix node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTLiteral node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTBooleanLiteral node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTNullLiteral node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTArguments node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTArgumentList node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTAllocationExpression node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTArrayDimsAndInits node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTStatement node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTLabeledStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTBlock node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTBlockStatement node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTLocalVariableDeclaration node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTEmptyStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTStatementExpression node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTSwitchStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTSwitchLabel node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTIfStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTWhileStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTDoStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTForStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTForInit node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTStatementExpressionList node, Object data)
    {
        //print(node, data);//TODO
        return print(node, data);
    }

    public Object visit(ASTForUpdate node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTBreakStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTContinueStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTReturnStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTThrowStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTSynchronizedStatement node, Object data)
    {
        return print(node, data);
    }

    public Object visit(ASTTryStatement node, Object data)
    {
        return print(node, data);
    }

    public void setList(ArrayList list)
    {
        this.list = list;
    }

    public ArrayList getList()
    {
        return list;
    }

}
