package utilities.log;
import java.io.PrintStream;

/**
 * A simple logging class.<P>
 */

public class Log {
  private static final boolean DEBUG = true;
  private static final boolean ERROR = DEBUG & true;
  private static final boolean WARN = DEBUG & true;
  private static final boolean INFO = DEBUG & true;
  private static final boolean LOG = DEBUG & true;
  private static LogInterface log_adaptor = new LogAdaptorLogWindow();
  //private static LogInterface log_adaptor = new LogAdaptorSystemOut();

  private Log() {
  }

  public static void log(String message) {
    if (LOG) {
      logLine(message);
    }
  }

  //  private static PrintStream getPrintStream() {
  //    return print_stream;
  //  }

  /**
   * Log an error
   */
  public static void error(String message) {
    if (ERROR) {
      logLine("Error: " + message);
    }
  }

  /**
   * Log a warning
   */
  public static void warn(String message) {
    if (WARN) {
      logLine("Warning: " + message);
    }
  }

  /**
   * Log an informational message
   */
  public static void info(String message) {
    if (INFO) {
      logLine("Information: " + message);
    }
  }

  /**
   * Log a plain message
   */
  private static void logLine(String message) {
    putString(message + "\n");
  }

  /**
   * Put out a partial line
   */
  private static void putString(String message) {
    log_adaptor.log(message);
  }

  public static void exception(Throwable e) {
    logLine("" + e);
    StackTraceElement[] ste = e.getStackTrace();
    for (int i = 0; i < ste.length; i++) {
      logLine("" + ste[i]);
    }
  }
  
  public static void stack() {
    try {
      throw new RuntimeException("Log stack trace");
    } catch (RuntimeException e) {
      exception(e);
    }
  }
}
